/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
import { Container, ContainerDefinition, Resource, SqlQuerySpec } from '@azure/cosmos';
import { AnnotatedResponse } from '../interceptor';
import { Omit } from '../types/internal';
import { TimeoutsOptions } from '../types/public';
export default class CosmosDbClient {
    private _endpoint;
    private _client;
    private _session?;
    private _retryOptions;
    static create(account: string, database: string, collection: string, key: string, collectionOptions: Omit<ContainerDefinition, 'id'>, retryOptions?: TimeoutsOptions): Promise<CosmosDbClient>;
    static getErrorRU(err: any): number | undefined;
    get containerRef(): string;
    constructor(endpoint: string, container: Container, retryOptions: TimeoutsOptions);
    documentRef(partition: string, id: string): string;
    createItem<T>(document: T): Promise<AnnotatedResponse<T & Resource>>;
    getItem<T>(id: string, partition: string): Promise<AnnotatedResponse<(T & Resource) | undefined>>;
    queryItemsArray<T>(query: SqlQuerySpec): Promise<AnnotatedResponse<T[]>>;
    queryItemsPage<T>(query: SqlQuerySpec, options: {
        continuation?: string;
        pageSize?: number;
    }): Promise<AnnotatedResponse<T[]>>;
    queryItemsIterator<T>(query: SqlQuerySpec): AsyncIterable<AnnotatedResponse<T>>;
    deleteItem(id: string, partition: string): Promise<AnnotatedResponse<void>>;
    replaceItem<T extends DocumentBase>(document: T, partition: string): Promise<AnnotatedResponse<T>>;
    createSproc(id: string, body: (...args: any[]) => void): Promise<AnnotatedResponse<any>>;
    replaceSproc(id: string, body: (...args: any[]) => void): Promise<AnnotatedResponse<any>>;
    executeSproc<T>(id: string, partition: string, ...params: string[]): Promise<AnnotatedResponse<T>>;
    private _wrap;
    private _transformResult;
    private _transformError;
    private _updateSession;
    private static _translateError;
    private static _scrubError;
    private static _getRu;
    private static _getContinuation;
    private static _isCosmosError;
    private static _shouldRetry;
}
export interface DocumentBase {
    id: string;
    _etag: string;
}
//# sourceMappingURL=cosmos.d.ts.map