"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url = __importStar(require("url"));
const cosmos_1 = require("@azure/cosmos");
const constants_1 = require("../constants");
const error_1 = __importStar(require("../error"));
const retry_1 = __importDefault(require("../utils/retry"));
const ERROR_RU = Symbol('CosmosDB Error RU');
const retryableNetworkingErrors = ['ECONNREFUSED', 'EAI_AGAIN'];
class CosmosDbClient {
    constructor(endpoint, container, retryOptions) {
        this._endpoint = endpoint;
        this._client = container;
        this._retryOptions = retryOptions;
    }
    static async create(account, database, collection, key, collectionOptions, retryOptions = constants_1.INTERNAL_RETRY_OPTIONS) {
        try {
            // TODO: look into reducing request timeout
            const connectionPolicy = {
                retryOptions: {
                    maxRetryAttemptCount: 0,
                    fixedRetryIntervalInMilliseconds: 1000,
                    maxWaitTimeInSeconds: 1000 // Just to make types happy
                }
            };
            const client = new cosmos_1.CosmosClient({
                endpoint: account,
                key,
                connectionPolicy,
                consistencyLevel: 'Session'
            });
            const { database: db } = await client.databases.createIfNotExists({
                id: database
            }, {
                offerThroughput: constants_1.DEFAULT_DATABASE_THROUGHPUT
            });
            const { container } = await db.containers.createIfNotExists(Object.assign({ id: collection }, collectionOptions));
            return new CosmosDbClient(account, container, retryOptions);
        }
        catch (err) {
            throw CosmosDbClient._translateError(err);
        }
    }
    static getErrorRU(err) {
        if (err && typeof err[ERROR_RU] === 'number') {
            return err[ERROR_RU];
        }
        return undefined;
    }
    get containerRef() {
        return url.resolve(this._endpoint, this._client.url);
    }
    documentRef(partition, id) {
        return url.resolve(this._endpoint, this._client.item(id, partition).url);
    }
    async createItem(document) {
        return await this._wrap(async () => this._client.items.create(document, { sessionToken: this._session }));
    }
    async getItem(id, partition) {
        return await this._wrap(async () => {
            try {
                return await this._client
                    .item(id, partition)
                    .read({ sessionToken: this._session });
            }
            catch (err) {
                if (CosmosDbClient._isCosmosError(err, 404)) {
                    return {
                        headers: err.headers,
                        body: undefined
                    };
                }
                throw err;
            }
        });
    }
    async queryItemsArray(query) {
        return await this._wrap(async () => this._client.items
            .query(query, {
            sessionToken: this._session
        })
            .fetchAll());
    }
    async queryItemsPage(query, options) {
        return await this._wrap(async () => this._client.items
            .query(query, {
            sessionToken: this._session,
            continuation: options.continuation,
            maxItemCount: options.pageSize
        })
            .fetchNext());
    }
    queryItemsIterator(query) {
        return __asyncGenerator(this, arguments, function* queryItemsIterator_1() {
            var e_1, _a;
            const iterator = this._client.items
                .query(query, {
                sessionToken: this._session
            })
                .getAsyncIterator()[Symbol.asyncIterator]();
            const modifiedIterator = {
                [Symbol.asyncIterator]: () => ({
                    next: async () => {
                        try {
                            const initialResult = await iterator.next();
                            if (!initialResult.value) {
                                // The spec allows the value to be omitted, but the
                                // Typescript type does not. Force it to behave with
                                // any.
                                return initialResult;
                            }
                            return {
                                value: this._transformResult(initialResult.value),
                                done: initialResult.done || false
                            };
                        }
                        catch (err) {
                            throw this._transformError(err);
                        }
                    }
                })
            };
            try {
                for (var modifiedIterator_1 = __asyncValues(modifiedIterator), modifiedIterator_1_1; modifiedIterator_1_1 = yield __await(modifiedIterator_1.next()), !modifiedIterator_1_1.done;) {
                    const result = modifiedIterator_1_1.value;
                    for (const [i, entry] of result.result.entries()) {
                        // Return the original result with the original RU consumption for the
                        // first entry, undefined for the rest.
                        yield yield __await({
                            ruConsumption: i === 0 ? result.ruConsumption : undefined,
                            result: entry
                        });
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (modifiedIterator_1_1 && !modifiedIterator_1_1.done && (_a = modifiedIterator_1.return)) yield __await(_a.call(modifiedIterator_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    async deleteItem(id, partition) {
        return await this._wrap(async () => {
            try {
                return Object.assign(Object.assign({}, (await this._client
                    .item(id, partition)
                    .delete({ sessionToken: this._session }))), { result: undefined });
            }
            catch (err) {
                if (CosmosDbClient._isCosmosError(err, 404)) {
                    return {
                        headers: err.headers,
                        result: undefined
                    };
                }
                throw err;
            }
        });
    }
    async replaceItem(document, partition) {
        return await this._wrap(async () => this._client.item(document.id, partition).replace(document, {
            accessCondition: {
                type: 'IfMatch',
                condition: document._etag
            },
            sessionToken: this._session
        }));
    }
    async createSproc(id, body) {
        return await this._wrap(async () => this._client.scripts.storedProcedures.create({ id, body }));
    }
    async replaceSproc(id, body) {
        return await this._wrap(async () => this._client.scripts.storedProcedure(id).replace({ id, body }));
    }
    async executeSproc(id, partition, ...params) {
        return await this._wrap(async () => this._client.scripts.storedProcedure(id).execute(partition, params));
    }
    async _wrap(operation) {
        return await retry_1.default(async () => {
            try {
                return this._transformResult(await operation());
            }
            catch (err) {
                throw this._transformError(err);
            }
        }, CosmosDbClient._shouldRetry, this._retryOptions);
    }
    _transformResult(response) {
        this._updateSession(response.headers);
        let result;
        if ('resource' in response) {
            result = response.resource;
        }
        else if ('resources' in response) {
            result = response.resources;
        }
        else {
            result = response.result;
        }
        return {
            ruConsumption: CosmosDbClient._getRu(response.headers),
            continuation: CosmosDbClient._getContinuation(response.headers),
            result
        };
    }
    _transformError(error) {
        // Cosmos DB errors contain the response headers for the underlying
        // request. We pull them off so we can get RU and session information.
        const headers = error && error.headers;
        this._updateSession(headers);
        return CosmosDbClient._translateError(error, CosmosDbClient._getRu(headers));
    }
    _updateSession(headers) {
        // Cosmos DB session tokens are maintained per collection and change
        // with each update to the database. Since the entire client is scoped
        // to a single collection, we simply pull the most recent session token
        // we've seen and use it for subsequent requests. This allows us to
        // support collections configured with session consistency while still
        // maintaining reasonable linearizability guarantees within scenarios.
        if (headers && headers[constants_1.SESSION_TOKEN_HEADER]) {
            this._session = headers[constants_1.SESSION_TOKEN_HEADER];
        }
    }
    static _translateError(err, ruConsumption) {
        // We scrub the error first for sensitive information
        const scrubbedError = CosmosDbClient._scrubError(err);
        let code;
        switch (scrubbedError && scrubbedError.code) {
            case 400:
                code = error_1.ErrorCode.DATABASE_INVALID_REQUEST;
                break;
            case 401:
                code = error_1.ErrorCode.DATABASE_INVALID_CREDENTIALS;
                break;
            case 403:
                code = error_1.ErrorCode.DATABASE_OPERATION_FORBIDDEN;
                break;
            case 404:
                code = error_1.ErrorCode.DATABASE_RESOURCE_NOT_FOUND;
                break;
            case 408:
                code = error_1.ErrorCode.DATABASE_SPROC_TIMED_OUT;
                break;
            case 409:
                code = error_1.ErrorCode.DATABASE_RESOURCE_ALREADY_EXISTS;
                break;
            case 412:
                code = error_1.ErrorCode.DATABASE_RESOURCE_CONCURRENCY_VIOLATION;
                break;
            case 413:
                code = error_1.ErrorCode.TASK_TOO_LARGE;
                break;
            case 429:
                code = error_1.ErrorCode.DATABASE_THROUGHPUT_EXCEEDED;
                break;
            case 449:
                code = error_1.ErrorCode.DATABASE_TRANSIENT_WRITE_FAILURE;
                break;
            case 500:
                code = error_1.ErrorCode.DATABASE_INTERNAL_ERROR;
                break;
            case 503:
                code = error_1.ErrorCode.DATABASE_UNAVAILABLE;
                break;
            case 'ENOTFOUND':
                code = error_1.ErrorCode.DATABASE_ACCOUNT_NOT_FOUND;
                break;
        }
        if (code) {
            const error = new error_1.default(code, scrubbedError);
            error[ERROR_RU] = ruConsumption;
            return error;
        }
        return scrubbedError;
    }
    static _scrubError(err) {
        // Sometimes authorization headers can be exposed via cosmos SDK errors. We
        // scrub this out to avoid it propagating downstream to users and getting
        // logged into external systems.
        if (err && err.requestHeaders && err.requestHeaders.authorization) {
            delete err.requestHeaders.authorization;
        }
        return err;
    }
    static _getRu(headers) {
        return headers && headers[constants_1.RU_HEADER]
            ? Number(headers[constants_1.RU_HEADER])
            : undefined;
    }
    static _getContinuation(headers) {
        return headers && headers[constants_1.CONTINUATION_HEADER];
    }
    static _isCosmosError(err, ...codes) {
        return err && codes.includes(err.code);
    }
    static _shouldRetry(err) {
        return (error_1.default.is(err, error_1.ErrorCode.DATABASE_TRANSIENT_WRITE_FAILURE, error_1.ErrorCode.DATABASE_UNAVAILABLE, error_1.ErrorCode.DATABASE_ACCOUNT_NOT_FOUND) ||
            (err && retryableNetworkingErrors.includes(err.code)));
    }
}
exports.default = CosmosDbClient;
//# sourceMappingURL=cosmos.js.map