"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncDelegator = (this && this.__asyncDelegator) || function (o) {
    var i, p;
    return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
};
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Version of {@link TaskClient} that has all of its operations automatically
 * scoped to a single task type.
 *
 * @public
 */
class ScopedTaskClient {
    /**
     * @internal
     */
    constructor(client, type) {
        this._client = client;
        this._type = type;
    }
    /**
     * Creates a task and saves it to the database.
     *
     * @param payload - Payload containing the data for the task
     * @param options - Options to configure the newly created task
     *
     * @public
     */
    async create(payload, options) {
        return this._client.create(this._type, payload, options);
    }
    /**
     * Retrieves the task of the given type with the provided id, if one exists.
     * If the task is not found, undefined is returned.
     *
     * @param id  - The task's id
     *
     * @public
     */
    async get(id) {
        return this._client.get(this._type, id);
    }
    /**
     * Retrieves all tasks of the given type, paged using the provided list
     * options.
     *
     * @param options - Options controlling which tasks to retrieve
     *
     * @public
     */
    async list(options = {}) {
        return this._client.list(this._type, options);
    }
    /**
     * Retrieves a single page from the database for tasks of the given type,
     * optionally starting from the provided continuation token.
     *
     * @param options - Options controlling which tasks to retrieve
     *
     * @public
     */
    async listPaged(options = {}) {
        return this._client.listPaged(this._type, options);
    }
    /**
     * Retrieves a all tasks of the given type with the entire payload omitted
     * by default. This is primarily useful if you have tasks with a large
     * amount of data in the payload that you don't need to see in the listed
     * results and you want to save cost/memory.
     *
     * @remarks
     *
     * Results are paged, filtered and sorted using the provided options. You
     * may also specify certain properties of the payload you want to see in the
     * returned task through the options.
     *
     * @param options - Options controlling which tasks to retrieve
     *
     * @public
     */
    async listSummary(options = {}) {
        return this._client.listSummary(this._type, options);
    }
    /**
     * Retrieves a single page of tasks of the given type with the entire payload
     * omitted by default, optionally starting from the provided continuation
     * token. This is primarily useful if you have tasks with a large amount of
     * data in the payload that you don't need to see in the listed results and
     * you want to save cost/memory.
     *
     * @remarks
     *
     * Results are paged, filtered and sorted using the provided options. You
     * may also specify certain properties of the payload you want to see in the
     * returned task through the options.
     *
     * @param type    - Task type
     * @param options - Options controlling which tasks to retrieve
     *
     * @public
     */
    async listSummaryPaged(options = {}) {
        return this._client.listSummaryPaged(this._type, options);
    }
    /**
     * Returns an async iterator for iterating over all tasks of the given type,
     * optionally filtered/sorted using the provided options object.
     *
     * @remarks
     *
     * _NOTE: Asynchronous iterators are a new feature of Javascript and are
     * only available without a polyfill starting in Node.js 10. You can make
     * use of async iterators in prior versions of Node.js by using TypeScript
     * or Babel to downcompile the code for you:
     * https://node.green/#ES2018-features-Asynchronous-Iterators_
     *
     * @param options - Options controlling which tasks to retrieve and the
     *                  order in which to receive them
     *
     * @public
     */
    iterate(options = {}) {
        return __asyncGenerator(this, arguments, function* iterate_1() {
            yield __await(yield* __asyncDelegator(__asyncValues(this._client.iterate(this._type, options))));
        });
    }
    /**
     * Returns an async iterator for iterating over all tasks of the given type
     * with the entire payload omitted by default. This is primarily useful if
     * you have tasks with a large amount of data in the payload that you don't
     * need to see in the listed results and you want to save cost/memory.
     *
     * @remarks
     *
     * Results are filtered and sorted using the provided options. You may also
     * specify certain properties of the payload you want to see in the returned
     * task through the options.
     *
     * _NOTE: Asynchronous iterators are a new feature of Javascript and are
     * only available without a polyfill starting in Node.js 10. You can make
     * use of async iterators in prior versions of Node.js by using TypeScript
     * or Babel to downcompile the code for you:
     * https://node.green/#ES2018-features-Asynchronous-Iterators_
     *
     * @param options - Options controlling which tasks to retrieve and the
     *                  order in which to receive them
     *
     * @public
     */
    iterateSummary(options = {}) {
        return __asyncGenerator(this, arguments, function* iterateSummary_1() {
            yield __await(yield* __asyncDelegator(__asyncValues(this._client.iterateSummary(this._type, options))));
        });
    }
    /**
     * Compute the number of tasks of the given type that currently exist,
     * optionally filtered to a custom group of tasks within the type.
     *
     * @param filter  - Query filter specifying which tasks within the provided
     *                  type to include
     */
    async count(filter) {
        return this._client.count(this._type, filter);
    }
    /**
     * Disables tasks of the scoped type, optionally filtered to a custom
     * group of tasks within the type.
     *
     * @param filter  - Query filter specifying which tasks within the scoped
     *                  type to disable
     *
     * @public
     */
    async disable(filter) {
        return this._client.disable(this._type, filter);
    }
    /**
     * Enables tasks of the scoped type, optionally filtered to a custom group
     * of tasks within the type.
     *
     * @param filter  - Query filter specifying which tasks within the scoped
     *                  type to enable
     *
     * @public
     */
    async enable(filter) {
        return this._client.enable(this._type, filter);
    }
    /**
     * Delete the task from the database. The operation is idempotent and will
     * succeed even if the task has already been deleted.
     *
     * @param id  - The task's id
     *
     * @public
     */
    async deleteOne(id) {
        return this._client.deleteOne(this._type, id);
    }
    /**
     * Deletes tasks of the scoped type, optionally filtered to a custom group
     * of tasks within the type.
     *
     * @param filter  - Query filter specifying which tasks within the scoped
     *                  type to delete
     *
     * @public
     */
    async delete(filter) {
        return this._client.delete(this._type, filter);
    }
    /**
     * Start listening for tasks of the scoped type.
     *
     * @param handler - Function to call with each task received for processing.
     *                  May be synchronous or asynchronous
     * @param options - Options for configuring the listener
     *
     * @public
     */
    listen(handler, options) {
        return this._client.listen(this._type, handler, options);
    }
}
exports.default = ScopedTaskClient;
//# sourceMappingURL=scoped.js.map