"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const messages_1 = __importDefault(require("./messages"));
/**
 * Generic error class that is thrown for any internal library errors.
 *
 * @public
 */
class IronTaskError extends Error {
    /**
     * @public
     */
    constructor(code, parentError) {
        super(messages_1.default[code]);
        /**
         * Static name for identifying that a task is of this type. This can be used
         * as an alternative to performing an `instanceof` check.
         *
         * @public
         */
        this.name = 'IronTaskError';
        this.code = code;
        this.parentError = parentError;
    }
    /**
     * Checks the provided error to see if it is a valid IronTaskError and has
     * one of the provided codes. If no codes are provided, it will just check
     * whether the error is an IronTaskError.
     *
     * @param error   - Arbitrary error to verify
     * @param codes   - List of error codes to check for. If the error does not
     *                  have one of the codes, this will return false. If there
     *                  are no error codes provided, the function will only
     *                  check whether this is a valid IronTaskError.
     *
     * @public
     */
    static is(error, ...codes) {
        if (!(error instanceof IronTaskError)) {
            return false;
        }
        if (codes.length > 0 && !codes.includes(error.code)) {
            return false;
        }
        return true;
    }
}
exports.default = IronTaskError;
//# sourceMappingURL=error.js.map