"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_1 = require("../client");
const error_1 = __importStar(require("../error"));
/**
 * @public
 */
class InterceptorProcessor {
    constructor(interceptors) {
        this._interceptors = interceptors;
    }
    async client(client, operation, ref, type, handler) {
        if (!this._interceptors.client) {
            const { result } = await handler();
            return result;
        }
        const context = {
            client,
            operation,
            ref,
            type
        };
        let result;
        let called = false;
        await this._interceptors.client(context, async () => {
            try {
                if (called) {
                    throw new error_1.default(error_1.ErrorCode.INTERCEPTOR_NEXT_FUNCTION_ALREADY_CALLED);
                }
                called = true;
                const response = await handler();
                context.ruConsumption = response.ruConsumption;
                result = response.result;
                return response.result;
            }
            catch (err) {
                context.ruConsumption = client_1.CosmosDbClient.getErrorRU(err);
                throw err;
            }
        });
        return result;
    }
    async task(task, operation, ref, handler) {
        if (!this._interceptors.task) {
            const { result } = await handler();
            return result;
        }
        const context = {
            task,
            operation,
            ref
        };
        let result;
        let called = false;
        await this._interceptors.task(context, async () => {
            try {
                if (called) {
                    throw new error_1.default(error_1.ErrorCode.INTERCEPTOR_NEXT_FUNCTION_ALREADY_CALLED);
                }
                called = true;
                const response = await handler();
                context.ruConsumption = response.ruConsumption;
                result = response.result;
                return response.result;
            }
            catch (err) {
                context.ruConsumption = client_1.CosmosDbClient.getErrorRU(err);
                throw err;
            }
        });
        return result;
    }
    async processing(listener, task, ref, handler) {
        if (!this._interceptors.processing) {
            await handler();
            return;
        }
        const context = {
            listener,
            task,
            ref
        };
        try {
            let called = false;
            await this._interceptors.processing(context, async () => {
                if (called) {
                    throw new error_1.default(error_1.ErrorCode.INTERCEPTOR_NEXT_FUNCTION_ALREADY_CALLED);
                }
                called = true;
                const response = await handler();
                context.error = response.error;
                context.delayMs = response.delayMs;
                return response.result;
            });
        }
        catch (_a) {
            // Errors here can only be thrown by the user code, so we swallow
            // them if they happen.
        }
    }
}
exports.default = InterceptorProcessor;
//# sourceMappingURL=processor.js.map