"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const symbols_1 = require("../symbols");
/**
 * Query expressions for creating Cosmos DB filters in operations such as {@link
 * TaskClient.list}, {@link TaskClient.listSummary}, {@link TaskClient.iterate}
 * and {@link TaskClient.iterateSummary}.
 *
 * @public
 */
var q;
(function (q) {
    /**
     * Absolute value
     *
     * @public
     */
    q.abs = (num) => [symbols_1.FnSymbol.ABS, num];
    /**
     * Arccosine trigonometric function
     *
     * @public
     */
    q.acos = (num) => [symbols_1.FnSymbol.ACOS, num];
    /**
     * Arcsine trigonometric function
     *
     * @public
     */
    q.asin = (num) => [symbols_1.FnSymbol.ASIN, num];
    /**
     * Arctangent trigonometric function
     *
     * @public
     */
    q.atan = (num) => [symbols_1.FnSymbol.ATAN, num];
    /**
     * Principal value of the arctangent of the second argument divided by the
     * first
     *
     * @public
     */
    q.atn2 = (num1, num2) => [
        symbols_1.FnSymbol.ATN2,
        num1,
        num2
    ];
    /**
     * Round up to the nearest integer.
     *
     * @public
     */
    q.ceiling = (num) => [
        symbols_1.FnSymbol.CEILING,
        num
    ];
    /**
     * Cosine trigonometric function
     *
     * @public
     */
    q.cos = (num) => [symbols_1.FnSymbol.COS, num];
    /**
     * Cotangent trigonometric function
     *
     * @public
     */
    q.cot = (num) => [symbols_1.FnSymbol.COT, num];
    /**
     * Convert radians to degrees
     *
     * @public
     */
    q.degrees = (num) => [
        symbols_1.FnSymbol.DEGREES,
        num
    ];
    /**
     * Take the exponential
     *
     * @public
     */
    q.exp = (num) => [symbols_1.FnSymbol.EXP, num];
    /**
     * Round down to the nearest integer
     *
     * @public
     */
    q.floor = (num) => [
        symbols_1.FnSymbol.FLOOR,
        num
    ];
    /**
     * Take the natural logarithm, or the logarithm with the optional provided
     * base.
     *
     * @public
     */
    q.log = (num, base) => [
        symbols_1.FnSymbol.LOG,
        num,
        base
    ];
    /**
     * Take the base 10 logarithm
     *
     * @public
     */
    q.log10 = (num) => [
        symbols_1.FnSymbol.LOG10,
        num
    ];
    /**
     * The constant pi (3.14159...)
     *
     * @public
     */
    q.pi = () => [symbols_1.FnSymbol.PI];
    /**
     * Raise the expression to the given exponent.
     *
     * @public
     */
    q.power = (expr, exponent) => [symbols_1.FnSymbol.POWER, expr, exponent];
    /**
     * Convert degrees to radians
     *
     * @public
     */
    q.radians = (num) => [
        symbols_1.FnSymbol.RADIANS,
        num
    ];
    /**
     * Round to the nearest integer
     *
     * @public
     */
    q.round = (num) => [
        symbols_1.FnSymbol.ROUND,
        num
    ];
    /**
     * Sine trigonometric function
     *
     * @public
     */
    q.sin = (num) => [symbols_1.FnSymbol.SIN, num];
    /**
     * Take the square root
     *
     * @public
     */
    q.sqrt = (num) => [symbols_1.FnSymbol.SQRT, num];
    /**
     * Raise the value to the 2nd power
     *
     * @public
     */
    q.square = (num) => [
        symbols_1.FnSymbol.SQUARE,
        num
    ];
    /**
     * Returns 1, 0 or -1 depending on whether the sign of the value is
     * positive, zero or negative, respectively.
     *
     * @public
     */
    q.sign = (num) => [symbols_1.FnSymbol.SIGN, num];
    /**
     * Tangent trigonometric function
     *
     * @public
     */
    q.tan = (num) => [symbols_1.FnSymbol.TAN, num];
    /**
     * Truncate the decimal portion of the value
     *
     * @public
     */
    q.trunc = (num) => [
        symbols_1.FnSymbol.TRUNC,
        num
    ];
    /**
     * Returns true if the value is an array
     *
     * @public
     */
    q.isArray = (val) => [
        symbols_1.FnSymbol.IS_ARRAY,
        val
    ];
    /**
     * Returns true if the value is a boolean
     *
     * @public
     */
    q.isBool = (val) => [
        symbols_1.FnSymbol.IS_BOOL,
        val
    ];
    /**
     * Returns true if the value is defined
     *
     * @public
     */
    q.isDefined = (val) => [
        symbols_1.FnSymbol.IS_DEFINED,
        val
    ];
    /**
     * Returns true if the value is null
     *
     * @public
     */
    q.isNull = (val) => [
        symbols_1.FnSymbol.IS_NULL,
        val
    ];
    /**
     * Returns true if the value is a number
     *
     * @public
     */
    q.isNumber = (val) => [
        symbols_1.FnSymbol.IS_NUMBER,
        val
    ];
    /**
     * Returns true if the value is an object
     *
     * @public
     */
    q.isObject = (val) => [
        symbols_1.FnSymbol.IS_OBJECT,
        val
    ];
    /**
     * Returns true if the value is a primitive value (string, number, boolean
     * or null)
     *
     * @public
     */
    q.isPrimitive = (val) => [
        symbols_1.FnSymbol.IS_PRIMITIVE,
        val
    ];
    /**
     * Returns true if the value is a string
     *
     * @public
     */
    q.isString = (val) => [
        symbols_1.FnSymbol.IS_STRING,
        val
    ];
    /**
     * Check if the first string contains the second
     *
     * @public
     */
    q.contains = (str1, str2) => [symbols_1.FnSymbol.CONTAINS, str1, str2];
    /**
     * Check if the first string ends with the second string
     *
     * @public
     */
    q.endsWith = (str1, str2) => [symbols_1.FnSymbol.ENDSWITH, str1, str2];
    /**
     * Returns the starting position of the second string within the first
     * string, or -1 if it is not found
     *
     * @public
     */
    q.indexOf = (str1, str2) => [symbols_1.FnSymbol.INDEX_OF, str1, str2];
    /**
     * Returns the left portion of the string up to the number of charaters
     * specified
     *
     * @public
     */
    q.left = (str, num) => [symbols_1.FnSymbol.LEFT, str, num];
    /**
     * Returns the length of the string
     *
     * @public
     */
    q.length = (str) => [
        symbols_1.FnSymbol.LENGTH,
        str
    ];
    /**
     * Convert the string to lower case
     *
     * @public
     */
    q.lower = (str) => [
        symbols_1.FnSymbol.LOWER,
        str
    ];
    /**
     * Trim whitespace from the left side of the string
     *
     * @public
     */
    q.ltrim = (str) => [
        symbols_1.FnSymbol.LTRIM,
        str
    ];
    /**
     * Replace instances of the search string with the replacement string in the
     * input string
     *
     * @public
     */
    q.replace = (str, search, replace) => [symbols_1.FnSymbol.REPLACE, str, search, replace];
    /**
     * Repeat the string the provided number of times
     *
     * @public
     */
    q.replicate = (str, num) => [symbols_1.FnSymbol.REPLICATE, str, num];
    /**
     * Reverse the characters of the string
     *
     * @public
     */
    q.reverse = (str) => [
        symbols_1.FnSymbol.REVERSE,
        str
    ];
    /**
     * Return the right portion of the string up to the number of characters
     * provided
     *
     * @public
     */
    q.right = (str, num) => [symbols_1.FnSymbol.RIGHT, str, num];
    /**
     * Trim whitespace from the right side of the string
     *
     * @public
     */
    q.rtrim = (str) => [
        symbols_1.FnSymbol.RTRIM,
        str
    ];
    /**
     * Check whether the first string starts with the second
     *
     * @public
     */
    q.startsWith = (str1, str2) => [symbols_1.FnSymbol.STARTSWITH, str1, str2];
    /**
     * Return the portion of the string from the start potition up to the length
     * provided
     *
     * @public
     */
    q.substring = (str, start, length) => [symbols_1.FnSymbol.SUBSTRING, str, start, length];
    /**
     * Convert the value to a string
     *
     * @public
     */
    q.toString = (val) => [
        symbols_1.FnSymbol.TO_STRING,
        val
    ];
    /**
     * Trim whitespace from both ends of the string
     *
     * @public
     */
    q.trim = (str) => [
        symbols_1.FnSymbol.TRIM,
        str
    ];
    /**
     * Convert the string to upper case
     *
     * @public
     */
    q.upper = (str) => [
        symbols_1.FnSymbol.UPPER,
        str
    ];
    /**
     * Concatenate all of the provided arrays together. Requires at least two
     * arrays to concat.
     *
     * @public
     */
    q.arrayConcat = (array1, array2, ...more) => [symbols_1.FnSymbol.ARRAY_CONCAT, array1, array2, more];
    /**
     * Check whether the array contains the item specified. If partial is
     * true, then the item is matched if it is a valid portion of one of the
     * values, rather than having to be the full value.
     *
     * @public
     */
    q.arrayContains = (array, item, partial) => [symbols_1.FnSymbol.ARRAY_CONTAINS, array, item, partial];
    /**
     * Compute the number of elements in the array
     *
     * @public
     */
    q.arrayLength = (array) => [
        symbols_1.FnSymbol.ARRAY_LENGTH,
        array
    ];
    /**
     * Return a subset of the array starting from the provided index and
     * optionally with the provided maximum length
     *
     * @public
     */
    q.arraySlice = (array, start, length) => [symbols_1.FnSymbol.ARRAY_SLICE, array, start, length];
    /**
     * Returns the distance between two GeoJSON points, polygons or linestrings
     *
     * @public
     */
    q.stDistance = (spatial1, spatial2) => [symbols_1.FnSymbol.ST_DISTANCE, spatial1, spatial2];
    /**
     * Check whether the first GeoJSON point polygon or linestring is inside of
     * the GeoJSON point, polygon or linestring in the second argument
     *
     * @public
     */
    q.stWithin = (inner, outer) => [symbols_1.FnSymbol.ST_WITHIN, inner, outer];
    /**
     * Check whether the two GeoJSON points, polyhons, or linestrings intersect
     * one another
     *
     * @public
     */
    q.stIntersects = (spatial1, spatial2) => [symbols_1.FnSymbol.ST_INTERSECTS, spatial1, spatial2];
    /**
     * Check whether the value is a valid GeoJSON point, polygon or linestring
     *
     * @public
     */
    q.stIsValid = (spatial) => [
        symbols_1.FnSymbol.ST_ISVALID,
        spatial
    ];
    /**
     * Check whether the value is a valid GeoJSON point, polygon or linestring,
     * returning an object with a `valid` boolean indicating whether the value
     * is valid and a `reason` property with a string describing the error if it
     * was not valid.
     *
     * @public
     */
    q.stIsValidDetailed = (spatial) => [symbols_1.FnSymbol.ST_ISVALIDDETAILED, spatial];
    /**
     * Boolean AND of two or more subexpressions
     *
     * @public
     */
    q.and = (...expressions) => [
        symbols_1.OpSymbol.AND,
        expressions
    ];
    /**
     * Boolean OR of two or more subexpressions
     *
     * @public
     */
    q.or = (...expressions) => [
        symbols_1.OpSymbol.OR,
        expressions
    ];
    /**
     * Boolean negation of the subexpression
     *
     * @public
     */
    q.not = (expression) => [
        symbols_1.OpSymbol.NOT,
        expression
    ];
    /**
     * Add the two numbers
     *
     * @public
     */
    q.add = (left, right) => [
        symbols_1.OpSymbol.ADD,
        left,
        right
    ];
    /**
     * Subtract the second number from the first
     *
     * @public
     */
    q.subtract = (left, right) => [symbols_1.OpSymbol.SUBTRACT, left, right];
    /**
     * Multiply the two numbers
     *
     * @public
     */
    q.multiply = (left, right) => [symbols_1.OpSymbol.MULTIPLY, left, right];
    /**
     * Divide the first number by the second
     *
     * @public
     */
    q.divide = (left, right) => [
        symbols_1.OpSymbol.DIVIDE,
        left,
        right
    ];
    /**
     * Compute the modulus of dividing the first number by the second
     *
     * @public
     */
    q.modulo = (left, right) => [
        symbols_1.OpSymbol.MODULO,
        left,
        right
    ];
    /**
     * Take the bitwise OR of the two numbers
     *
     * @public
     */
    q.bitwiseOr = (left, right) => [symbols_1.OpSymbol.BITWISE_OR, left, right];
    /**
     * Take the bitwise AND of the two numbers
     *
     * @public
     */
    q.bitwiseAnd = (left, right) => [symbols_1.OpSymbol.BITWISE_AND, left, right];
    /**
     * Take the bitwise exclusive or of the two numbers
     *
     * @public
     */
    q.xor = (left, right) => [
        symbols_1.OpSymbol.XOR,
        left,
        right
    ];
    /**
     * Shift the bits of the number to the left by the given amount
     *
     * @public
     */
    q.leftShift = (left, right) => [symbols_1.OpSymbol.LEFT_SHIFT, left, right];
    /**
     * Shift the bits of the number to the right by the given amount
     *
     * @public
     */
    q.rightShift = (left, right) => [symbols_1.OpSymbol.RIGHT_SHIFT, left, right];
    /**
     * Shift the bits of the number to the right by the given amount, filling
     * the new leftmost bits with zeros
     *
     * @public
     */
    q.zeroFillRightShift = (left, right) => [symbols_1.OpSymbol.ZF_RIGHT_SHIFT, left, right];
    /**
     * Check whether the two values are equivalent
     *
     * @public
     */
    q.equal = (left, right) => [
        symbols_1.OpSymbol.EQUAL,
        left,
        right
    ];
    /**
     * Check whether the two values are not equivalent
     *
     * @public
     */
    q.notEqual = (left, right) => [
        symbols_1.OpSymbol.NOT_EQUAL,
        left,
        right
    ];
    /**
     * Check whether the first number is greater than the second
     *
     * @public
     */
    q.greaterThan = (left, right) => [symbols_1.OpSymbol.GREATER_THAN, left, right];
    /**
     * Check whether the first number is greater than or equal to the second
     *
     * @public
     */
    q.greaterThanOrEqual = (left, right) => [symbols_1.OpSymbol.GREATER_THAN_OR_EQUAL, left, right];
    /**
     * Check whether the first number is less than the second
     *
     * @public
     */
    q.lessThan = (left, right) => [symbols_1.OpSymbol.LESS_THAN, left, right];
    /**
     * Check whether the first number is less than or equal to the second
     *
     * @public
     */
    q.lessThanOrEqual = (left, right) => [symbols_1.OpSymbol.LESS_THAN_OR_EQUAL, left, right];
    /**
     * Return the first value if defined, or the second value if the first value
     * if the first value is undefined
     *
     * @public
     */
    q.coalesce = (first, second) => [symbols_1.OpSymbol.COALESCE, first, second];
    /**
     * Concatenate two or more strings together into a single string
     *
     * @public
     */
    q.concat = (left, right, ...more) => [symbols_1.OpSymbol.CONCATENATE, left, right, more];
    /**
     * If the condition is true, return the second argument. If it is false,
     * return the third.
     *
     * @public
     */
    q.ternary = (condition, ifTrue, ifFalse) => [symbols_1.OpSymbol.TERNARY, condition, ifTrue, ifFalse];
    /**
     * Check if the value is equal to one of the array of candidates in the
     * second argument
     *
     * @public
     */
    q.inOp = (value, candidates) => [
        symbols_1.OpSymbol.IN,
        value,
        candidates
    ];
})(q || (q = {}));
exports.default = q;
//# sourceMappingURL=query.js.map