"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const expression_1 = __importDefault(require("./expression"));
const projection_1 = __importDefault(require("./projection"));
/**
 * Build a Cosmos DB SQL query from the following options. Defaults to selecting
 * all data from every document in the collection with no sort.
 *
 * @param config Object representation of the query to build
 */
function build(config) {
    let paramIndex = 0;
    const paging = config.offset !== undefined && config.limit !== undefined
        ? {
            query: 'OFFSET @offset LIMIT @limit',
            parameters: [
                {
                    name: '@offset',
                    value: config.offset
                },
                {
                    name: '@limit',
                    value: config.limit
                }
            ]
        }
        : undefined;
    const projection = config.rawProjection
        ? config.rawProjection
        : config.projection !== undefined
            ? projection_1.default(config.projection)
            : '*';
    const filter = config.filter !== undefined
        ? expression_1.default(config.filter, paramIndex)
        : undefined;
    if (filter) {
        paramIndex += filter.parameters.length;
    }
    const sort = config.sort !== undefined
        ? // Due to the fact that sort fields must be raw projections for now, we
            // mark the expression as a projection.
            // TODO: update this if Cosmos DB ever supports expressions in order by
            expression_1.default(config.sort, paramIndex, true)
        : undefined;
    return {
        query: `SELECT ${projection}
            FROM c
            ${filter ? `WHERE ${filter.query}` : ''}
            ${sort
            ? `ORDER BY ${sort.query} ${config.sortOrder === 'DESC' ? 'DESC' : 'ASC'}`
            : ''}
            ${paging ? paging.query : ''}`,
        parameters: [
            ...(filter ? filter.parameters : []),
            ...(sort ? sort.parameters : []),
            ...(paging ? paging.parameters : [])
        ]
    };
}
exports.default = build;
//# sourceMappingURL=query.js.map