"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
// Logical operators
/**
 * @public
 */
// tslint:disable:naming-convention
var OpSymbol;
(function (OpSymbol) {
    /**
     * @public
     */
    OpSymbol.AND = Symbol('AND');
    /**
     * @public
     */
    OpSymbol.OR = Symbol('OR');
    /**
     * @public
     */
    OpSymbol.NOT = Symbol('NOT');
    // Mathematical operators
    /**
     * @public
     */
    OpSymbol.ADD = Symbol('+');
    /**
     * @public
     */
    OpSymbol.SUBTRACT = Symbol('-');
    /**
     * @public
     */
    OpSymbol.MULTIPLY = Symbol('*');
    /**
     * @public
     */
    OpSymbol.DIVIDE = Symbol('/');
    /**
     * @public
     */
    OpSymbol.MODULO = Symbol('%');
    // Bitwise operators
    /**
     * @public
     */
    OpSymbol.BITWISE_OR = Symbol('|');
    /**
     * @public
     */
    OpSymbol.BITWISE_AND = Symbol('&');
    /**
     * @public
     */
    OpSymbol.XOR = Symbol('^');
    /**
     * @public
     */
    OpSymbol.LEFT_SHIFT = Symbol('<<');
    /**
     * @public
     */
    OpSymbol.RIGHT_SHIFT = Symbol('>>');
    /**
     * @public
     */
    OpSymbol.ZF_RIGHT_SHIFT = Symbol('>>>');
    // Comparison operators
    /**
     * @public
     */
    OpSymbol.EQUAL = Symbol('=');
    /**
     * @public
     */
    OpSymbol.NOT_EQUAL = Symbol('<>');
    /**
     * @public
     */
    OpSymbol.GREATER_THAN = Symbol('>');
    /**
     * @public
     */
    OpSymbol.GREATER_THAN_OR_EQUAL = Symbol('>=');
    /**
     * @public
     */
    OpSymbol.LESS_THAN = Symbol('<');
    /**
     * @public
     */
    OpSymbol.LESS_THAN_OR_EQUAL = Symbol('<=');
    /**
     * @public
     */
    OpSymbol.COALESCE = Symbol('??');
    // String operators
    /**
     * @public
     */
    OpSymbol.CONCATENATE = Symbol('||');
    // Ternary operators
    /**
     * @public
     */
    OpSymbol.TERNARY = Symbol('?');
    // Miscellaneous operators
    /**
     * @public
     */
    OpSymbol.IN = Symbol('IN');
    /**
     * @internal
     */
    function getOp(sym) {
        switch (sym) {
            // Logical operators
            case OpSymbol.AND:
                return 'AND';
            case OpSymbol.OR:
                return 'OR';
            case OpSymbol.NOT:
                return 'NOT';
            // Mathematical operators
            case OpSymbol.ADD:
                return '+';
            case OpSymbol.SUBTRACT:
                return '-';
            case OpSymbol.MULTIPLY:
                return '*';
            case OpSymbol.DIVIDE:
                return '/';
            case OpSymbol.MODULO:
                return '%';
            // Bitwise operators
            case OpSymbol.BITWISE_OR:
                return '|';
            case OpSymbol.BITWISE_AND:
                return '&';
            case OpSymbol.XOR:
                return '^';
            case OpSymbol.LEFT_SHIFT:
                return '<<';
            case OpSymbol.RIGHT_SHIFT:
                return '>>';
            case OpSymbol.ZF_RIGHT_SHIFT:
                return '>>>';
            // Comparison operators
            case OpSymbol.EQUAL:
                return '=';
            case OpSymbol.NOT_EQUAL:
                return '<>';
            case OpSymbol.GREATER_THAN:
                return '>';
            case OpSymbol.GREATER_THAN_OR_EQUAL:
                return '>=';
            case OpSymbol.LESS_THAN:
                return '<';
            case OpSymbol.LESS_THAN_OR_EQUAL:
                return '<=';
            case OpSymbol.COALESCE:
                return '??';
            // String operators
            case OpSymbol.CONCATENATE:
                return '||';
            // Miscellaneous operators
            case OpSymbol.IN:
                return 'IN';
        }
    }
    OpSymbol.getOp = getOp;
})(OpSymbol || (OpSymbol = {}));
exports.default = OpSymbol;
//# sourceMappingURL=operators.js.map