"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const query_1 = require("../query");
const properties_1 = __importDefault(require("./properties"));
const BASE_PROJECTION = [
    properties_1.default.id,
    query_1.p.str('_etag'),
    query_1.p.num('ttl'),
    query_1.p.obj('config'),
    query_1.p.num('_ts')
];
/**
 * Computes the overall projection for a task summary, given the projections for
 * the payload within the task.
 *
 * @param projections Property projections to include from the payload
 */
function summaryProjection(projections) {
    // If there is no projection specified
    if (!projections) {
        return BASE_PROJECTION;
    }
    for (const projection of projections) {
        if (projection[1] !== properties_1.default.payload()[1]) {
            throw new TypeError('Only properties inside of the payload can be projected');
        }
    }
    return [...BASE_PROJECTION, ...projections];
}
exports.summaryProjection = summaryProjection;
/**
 * Produces a filter for tasks of the given type, with optional additional
 * filters applied.
 *
 * @param type The task's type
 * @param filter Optional additional filter to apply to the task
 */
function typeFilter(type, filter) {
    return filter !== undefined
        ? query_1.q.and(query_1.q.equal(properties_1.default.type, type), filter)
        : query_1.q.equal(properties_1.default.type, type);
}
exports.typeFilter = typeFilter;
//# sourceMappingURL=query.js.map