"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = __importStar(require("util"));
const interceptor_1 = __importDefault(require("../interceptor"));
const data_1 = __importDefault(require("./data"));
/**
 * Representation of a task that cannot have its payload saved because it is
 * produced from operations that only provide a partial copy of the payload,
 * such as {@link TaskClient.listSummary} and {@link TaskClient.iterateSummary}.
 *
 * @typeParam T   - Type of the task's payload data. Can be any type, but the
 *                  data should be directly serializable to JSON. This type can
 *                  also be tailored to the data projected by the operation that
 *                  produced this task.
 *
 * @public
 */
class ReadonlyTaskImpl {
    constructor(data, interceptor) {
        this._data = data;
        this._interceptor = interceptor;
    }
    static create(client, interceptor, document) {
        return new ReadonlyTaskImpl(new data_1.default(client, document, false), interceptor);
    }
    get id() {
        return this._data.id;
    }
    get type() {
        return this._data.type;
    }
    get status() {
        return this._data.status;
    }
    get enabled() {
        return this._data.enabled;
    }
    get createTime() {
        return this._data.createTime;
    }
    get nextRunTime() {
        return this._data.nextRunTime;
    }
    get lastUpdatedTime() {
        return this._data.lastUpdatedTime;
    }
    get lastRun() {
        return this._data.lastRun;
    }
    get currentRunStartTime() {
        return this._data.currentRunStartTime;
    }
    get deliveries() {
        return this._data.deliveries;
    }
    get attempts() {
        return this._data.attempts;
    }
    get runs() {
        return this._data.runs;
    }
    get payload() {
        return this._data.payload;
    }
    get interval() {
        return this._data.interval;
    }
    get lastRunTime() {
        return this._data.lastRunTime;
    }
    async enable() {
        await this._interceptor.task(this, interceptor_1.default.TaskOperation.Enable, this._data.ref, async () => this._data.patch(() => ({ enabled: true })));
    }
    async disable() {
        await this._interceptor.task(this, interceptor_1.default.TaskOperation.Disable, this._data.ref, async () => this._data.patch(() => ({ enabled: false })));
    }
    async delete() {
        await this._interceptor.task(this, interceptor_1.default.TaskOperation.Delete, this._data.ref, async () => this._data.delete());
    }
    toJSON() {
        return this._data.toJSON();
    }
    // We cast as any so that types behave outside of this library
    [util.inspect.custom](depth, opts) {
        return this.toJSON();
    }
}
exports.default = ReadonlyTaskImpl;
//# sourceMappingURL=readonly.js.map