"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Enumeration of all of the possible outcomes when processing a task.
 *
 * @public
 */
var ProcessingResult;
(function (ProcessingResult) {
    /**
     * Processing finished successfully
     *
     * @public
     */
    ProcessingResult["Complete"] = "complete";
    /**
     * Processing failed permanently or retried too many times
     *
     * @public
     */
    ProcessingResult["Fail"] = "fail";
    /**
     * Processing failed but should be attempted again
     *
     * @public
     */
    ProcessingResult["Retry"] = "retry";
    /**
     * Processing should be continued at a later time. Not considered a failure
     *
     * @public
     */
    ProcessingResult["Defer"] = "defer";
    /**
     * Processing stopped without finishing. Generally used to finish disabling
     * a task or when destroying a listener.
     *
     * @public
     */
    ProcessingResult["Release"] = "release";
    /**
     * Processing was stopped without informing the database. Generally used
     * when there is an issue communicating with the database when trying to
     * finish processing.
     *
     * @public
     */
    ProcessingResult["ForceRelease"] = "forceRelease";
    /**
     * Processing stopped because the handler lost its lock on the task.
     *
     * @public
     */
    ProcessingResult["LockLost"] = "lockLost";
    /**
     * The task was deleted by the processor.
     *
     * @public
     */
    ProcessingResult["Delete"] = "delete";
})(ProcessingResult = exports.ProcessingResult || (exports.ProcessingResult = {}));
/**
 * Enumeration of different representations of the current processing status of
 * a task.
 *
 * @public
 */
var TaskStatus;
(function (TaskStatus) {
    /**
     * The task is not currently running but is scheduled to run at a future
     * time.
     *
     * @public
     */
    TaskStatus["Scheduled"] = "scheduled";
    /**
     * The task is not currently running and its next scheduled run time has
     * already passed.
     *
     * @public
     */
    TaskStatus["Pending"] = "pending";
    /**
     * The task is currently being processed.
     *
     * @public
     */
    TaskStatus["Running"] = "running";
    /**
     * The task completed its most recent run successfully and has no future
     * runs scheduled.
     *
     * @public
     */
    TaskStatus["Completed"] = "completed";
    /**
     * The task failed its most recent run and has no future runs scheduled.
     *
     * @public
     */
    TaskStatus["Failed"] = "failed";
    /**
     * The task has been disabled, but a processor is still running the task
     * from a previous run.
     *
     * @public
     */
    TaskStatus["Disabling"] = "disabling";
    /**
     * The task has been disabled and is not running. The task will not run
     * again unless it is re-enabled.
     *
     * @public
     */
    TaskStatus["Disabled"] = "disabled";
})(TaskStatus = exports.TaskStatus || (exports.TaskStatus = {}));
/**
 * Representation of the current status of processing an {@link ActiveTask}.
 *
 * @public
 */
var ProcessingState;
(function (ProcessingState) {
    /**
     * The handler is currently processing the task.
     *
     * @public
     */
    ProcessingState[ProcessingState["Active"] = 0] = "Active";
    /**
     * The handler is currently trying to finish processing of the task.
     *
     * @public
     */
    ProcessingState[ProcessingState["Finishing"] = 1] = "Finishing";
    /**
     * The handler has finished processing the task.
     *
     * @public
     */
    ProcessingState[ProcessingState["Finished"] = 2] = "Finished";
    /**
     * The handler lost its lock on the task while processing.
     *
     * @public
     */
    ProcessingState[ProcessingState["LockLost"] = 3] = "LockLost";
})(ProcessingState = exports.ProcessingState || (exports.ProcessingState = {}));
//# sourceMappingURL=types.js.map