/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * Options for configuring retries for an operation. Copied from `@types/retry`
 * to avoid requiring the type in the package.
 *
 * @public
 */
export interface TimeoutsOptions {
    /**
     * The exponential factor to use.
     *
     * @defaultValue 2
     *
     * @public
     */
    factor?: number;
    /**
     * The number of milliseconds before starting the first retry.
     *
     * @defaultValue 1000
     *
     * @public
     */
    minTimeout?: number;
    /**
     * The maximum number of milliseconds between two retries.
     *
     * @defaultValue Infinity
     *
     * @public
     */
    maxTimeout?: number;
    /**
     * Randomizes the timeouts by multiplying a factor between 1-2.
     *
     * @defaultValue false
     *
     * @public
     */
    randomize?: boolean;
    /**
     * The maximum amount of times to retry the operation.
     *
     * @defaultValue 10
     *
     * @public
     */
    retries?: number;
}
export interface TaskEnableOptions {
    /**
     * Enable a recurring task and recompute next run time
     *
     * @defaultValue false
     *
     * @public
     */
    recomputeNextRunTime?: boolean;
}
//# sourceMappingURL=public.d.ts.map