"use strict";
/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cron = __importStar(require("cron-parser"));
const error_1 = __importStar(require("../error"));
const isValidNextRun_1 = __importDefault(require("./isValidNextRun"));
/**
 * Computes the unix ms epoch when the task should be run next from the interval,
 * if one is provided. If no time is returned, the task should not be run again.
 *
 * @param interval      Number of milliseconds between runs or cron string.
 * @param previousStart If there was a previous run, the time when that run
 *                      started
 * @param lastRunTime   The last valid time nextRunTime can be set to.
 */
function computeNextRun(interval, previousStart, lastRunTime) {
    // Cron strings are treated the same way regardless of whether this is a
    // first run or not. We just compute the next valid time and use it.
    if (typeof interval === 'string') {
        try {
            return isValidNextRun_1.default(cron
                .parseExpression(interval, {
                currentDate: previousStart || Date.now()
            })
                .next()
                .getTime(), lastRunTime);
        }
        catch (e) {
            throw new error_1.default(error_1.ErrorCode.INVALID_CRON_STRING_INTERVAL, e);
        }
    }
    // For non-cron strings, the first run is always the current time by default
    if (!previousStart) {
        return isValidNextRun_1.default(Date.now(), lastRunTime);
    }
    // If there is an interval and this is not the first run, just compute the
    // next run by adding the interval to the current time.
    if (interval) {
        return isValidNextRun_1.default(previousStart + interval, lastRunTime);
    }
    // If we get here, it is not the first run and there is no interval, so we
    // have no more runs to do.
    return undefined;
}
exports.default = computeNextRun;
//# sourceMappingURL=computeNextRun.js.map