/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
import { TimeoutsOptions } from '../types/public';
/**
 * Attempt a promise-based operation with retries built in.
 *
 * @param operation     Operation to retry. It is passed the zero-indexed
 *                      attempt number and whether this is the last try.
 * @param shouldRetry   Function that takes the error that was thrown and
 *                      determines whether it should be retried. Defaults to
 *                      always retry.
 * @param options       Configuration options for attempts and timeouts from the
 *                      `retry` library.
 */
export default function retry<T>(operation: (attempt: number, lastTry: boolean) => Promise<T>, shouldRetry?: (err: any) => boolean, options?: TimeoutsOptions): Promise<T>;
export interface Attempt {
    startTime: Date;
    endTime: Date;
    durationMs: number;
    error?: any;
}
//# sourceMappingURL=retry.d.ts.map